﻿using System;
using System.Collections.Generic;
using System.Text;

namespace 自动实现属性Demo
{
    class Program
    {
        static void Main(string[] args)
        {

        }
    }

    /// <summary>
    /// 自定义类
    /// </summary>
    public class Person
    {
        // C# 3之前我们定义属性时，一般会像下面这样去定义
        // 首先会先定义私有字段，再定义属性来对字段进行访问
        //private string _name;
        //public string Name
        //{
        //    get { return _name; }
        //    set { _name = value; }
        //}

        // C# 3之后有自动实现的属性之后
        // 对于不需要额外验证的属性，就可以用自动实现的属性对属性的定义进行简化
        // 不再需要额外定义一个私有字段了,
        // 不定义私有字段并不是此时没有了私有字段，只是编译器帮我们生成一个匿名的私有字段，不需要我们在代码中写出
        // 减少我们书写的代码
        // 下面就是用自动实现的属性来定义的一个属性，其效果等效于上面属性的定义，不过比之前更加简洁了

        /// <summary>
        ///  姓名
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 年龄
        /// </summary>
        public int Age { get; private set; }

        /// <summary>
        ///  自定义构造函数
        /// </summary>
        /// <param name="name"></param>
        public Person(string name)
        {
            Name = name;
        }
    }

    /// <summary>
    ///  在结构体使用自动属性
    /// </summary>
    public struct TestPerson
    {
        // 自动属性
        public string Name { get; set; }

        // 在结构中所有构造函数都需要显示地调用无参数构造函数this()，
        // 否则会出现编译错误
        // 只有调用了无参数构造函数，编译器才知道所有字段都被赋值了
        public TestPerson(string name)
            : this()
        {
            this.Name = name;
        }
    }
}
