﻿using System.Collections.Generic;

namespace 对象集合初始化器Demo
{
    class Program
    {
        static void Main(string[] args)
        {
            #region 对象初始化演示
            // 在C# 3.0之前，我们可能会使用下面方式来初始化对象
            Person person1 = new Person();
            person1.Name = "learning hard";
            person1.Age = 25;

            Person person2 = new Person("learning hard");
            person2.Age = 25;

            // 如果类没有无参的构造函数就会出现编译时错误
            // 因为下面的语句是调用无参构造函数来对类中的字段进行初始化的
            // 大括号部分就是对象初始化程序
            Person person3 = new Person { Name = "learning hard", Age = 25 };

            // 下面代码和上面代码是等价的，只不过上面省略了构造函数的圆括号而已
            Person person4 = new Person() { Name = "learning hard", Age = 25 };

            Person person5 = new Person("learning hard") { Age = 25 };

            #endregion 
            #region 集合初始化演示

            // C# 3.0之前初始化集合使用的代码
            List<string> names = new List<string>();
            names.Add("learning hard1");
            names.Add("learning hard2");
            names.Add("learning hard3");

            // 有了C# 3.0中集合初始化特性之后，就可以简化代码
            // 同时下面也使用了隐式类型（使用了var关键字）
            var newnames = new List<string>
            {
                "learning hard1","learning hard2", "learning hard3"
            };
            #endregion 
        }
    }

    /// <summary>
    /// 自定义类
    /// </summary>
    public class Person
    {
        /// <summary>
        ///  姓名
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 年龄
        /// </summary>
        public int Age { get; set; }

        /// <summary>
        ///  定义无参的构造函数
        ///  如果类中自定义了带参数的构造函数,则编译不会生成默认的构造函数
        ///  如果没有默认的构造函数,则使用对象初始化时就会报错说没有实现无参的构造函数
        /// </summary>
        public Person()
        {
        }

        /// <summary>
        ///  自定义构造函数
        /// </summary>
        /// <param name="name"></param>
        public Person(string name)
        {
            Name = name;
        }
    }
}

